/**
 * Adapted from ReactCSSTransitionGroup.js by Facebook
 *
 * @providesModule ReactReactCSSTransitionReplace
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(_x, _x2, _x3) { var _again = true; _function: while (_again) { var object = _x, property = _x2, receiver = _x3; desc = parent = getter = undefined; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x = parent; _x2 = property; _x3 = receiver; _again = true; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _reactLibObjectAssign = require('react/lib/Object.assign');

var _reactLibObjectAssign2 = _interopRequireDefault(_reactLibObjectAssign);

var _reactAddonsTransitionGroup = require('react-addons-transition-group');

var _reactAddonsTransitionGroup2 = _interopRequireDefault(_reactAddonsTransitionGroup);

var _ReactCSSTransitionReplaceChild = require('./ReactCSSTransitionReplaceChild');

var _ReactCSSTransitionReplaceChild2 = _interopRequireDefault(_ReactCSSTransitionReplaceChild);

var reactCSSTransitionReplaceChild = _react2['default'].createFactory(_ReactCSSTransitionReplaceChild2['default']);

function createTransitionTimeoutPropValidator(transitionType) {
  var timeoutPropName = 'transition' + transitionType + 'Timeout';
  var enabledPropName = 'transition' + transitionType;

  return function (props) {
    // If the transition is enabled
    if (props[enabledPropName]) {
      // If no timeout duration is provided
      if (!props[timeoutPropName]) {
        return new Error(timeoutPropName + ' wasn\'t supplied to ReactCSSTransitionReplace: ' + 'this can cause unreliable animations and won\'t be supported in ' + 'a future version of React. See ' + 'https://fb.me/react-animation-transition-group-timeout for more ' + 'information.');

        // If the duration isn't a number
      } else if (typeof props[timeoutPropName] !== 'number') {
          return new Error(timeoutPropName + ' must be a number (in milliseconds)');
        }
    }
  };
}

var ReactCSSTransitionReplace = (function (_React$Component) {
  _inherits(ReactCSSTransitionReplace, _React$Component);

  function ReactCSSTransitionReplace() {
    var _this = this;

    _classCallCheck(this, ReactCSSTransitionReplace);

    _get(Object.getPrototypeOf(ReactCSSTransitionReplace.prototype), 'constructor', this).apply(this, arguments);

    this.state = {
      currentChild: _react2['default'].Children.only(this.props.children),
      nextChild: null,
      height: 'auto'
    };

    this._childLeft = function () {
      // Swap the children after the current child left.
      _this.setState({
        currentChild: _this.state.nextChild,
        nextChild: null
      });
    };

    this._childEntered = function () {
      // The height animation would have finished, so switch back to auto.
      if (_this.props.transitionHeight) {
        _this.setState({
          height: 'auto'
        });
      }
    };

    this._wrapChild = function (child) {
      // We need to provide this childFactory so that
      // ReactCSSTransitionReplaceChild can receive updates to name,
      // enter, and leave while it is leaving.
      return reactCSSTransitionReplaceChild({
        name: _this.props.transitionName,
        appear: _this.props.transitionAppear,
        enter: _this.props.transitionEnter,
        leave: _this.props.transitionLeave,
        appearTimeout: _this.props.transitionAppearTimeout,
        enterTimeout: _this.props.transitionEnterTimeout,
        leaveTimeout: _this.props.transitionLeaveTimeout,
        onEntered: _this._childEntered,
        onLeft: _this._childLeft
      }, child);
    };
  }

  _createClass(ReactCSSTransitionReplace, [{
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var _this2 = this;

      var nextChild = nextProps.children ? _react2['default'].Children.only(nextProps.children) : null;
      var currentChild = this.state.currentChild;

      if (currentChild && nextChild && nextChild.key === currentChild.key) {
        // Nothing changed, but we are re-rendering so update the currentChild.
        return this.setState({
          currentChild: nextChild
        });
      }

      var transitionHeight = nextProps.transitionHeight;
      var currentHeight = transitionHeight ? _reactDom2['default'].findDOMNode(this.refs.container).offsetHeight : 'auto';

      // The child was removed, so animate out.
      if (!nextChild) {
        return this.setState({
          height: currentHeight,
          currentChild: null
        }, function () {
          return transitionHeight && _this2.setState({ height: 0 });
        });
      }

      // The child was replaced or added, setting a nextChild will eventually animate it in.
      this.setState({
        height: currentHeight,
        currentChild: null,
        nextChild: nextChild
      });
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      var nextChild = this.refs.nextChild;

      // If there is a next child we'll be animating it in soon, so change to its height.
      if (nextChild && nextChild.offsetHeight !== this.state.height) {
        this.setState({
          height: nextChild.offsetHeight
        });
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props;
      var style = _props.style;
      var _props$className = _props.className;
      var className = _props$className === undefined ? '' : _props$className;

      var nextChildShadow = undefined;

      if (this.props.transitionHeight) {
        if (this.state.nextChild) {
          nextChildShadow = _react2['default'].createElement(
            'div',
            { ref: 'nextChild', style: { position: 'absolute', visibility: 'hidden' } },
            this.state.nextChild
          );
        }

        var animatingHeight = this.state.height !== 'auto';

        if (animatingHeight) {
          className = className + ' ' + this.props.transitionName + '-height';
        }

        style = (0, _reactLibObjectAssign2['default'])(style || {}, {
          overflow: animatingHeight ? 'hidden' : 'visible',
          height: this.state.height,
          display: 'block'
        });
      }

      return _react2['default'].createElement(
        'span',
        null,
        nextChildShadow,
        _react2['default'].createElement(
          _reactAddonsTransitionGroup2['default'],
          _extends({ ref: 'container' }, this.props, { childFactory: this._wrapChild,
            style: style, className: className }),
          this.state.currentChild
        )
      );
    }
  }], [{
    key: 'propTypes',
    value: {
      transitionName: _react2['default'].PropTypes.string.isRequired,
      transitionAppear: _react2['default'].PropTypes.bool,
      transitionEnter: _react2['default'].PropTypes.bool,
      transitionLeave: _react2['default'].PropTypes.bool,
      transitionHeight: _react2['default'].PropTypes.bool,
      transitionAppearTimeout: createTransitionTimeoutPropValidator('Appear'),
      transitionEnterTimeout: createTransitionTimeoutPropValidator('Enter'),
      transitionLeaveTimeout: createTransitionTimeoutPropValidator('Leave')
    },
    enumerable: true
  }, {
    key: 'defaultProps',
    value: {
      transitionAppear: false,
      transitionEnter: true,
      transitionLeave: true,
      transitionHeight: true
    },
    enumerable: true
  }]);

  return ReactCSSTransitionReplace;
})(_react2['default'].Component);

exports['default'] = ReactCSSTransitionReplace;
module.exports = exports['default'];